window.weixinReadyPromise = new Promise((resolve, reject) => {
  if (!window.navigator.userAgent.toLowerCase().includes('micromessenger')) {
    return resolve('other')
  }
  if (!window.wx) { alert('请引入微信相关文件') }
  window.wx.config({
    debug: false,
    appId: '',
    timestamp: 1,
    nonceStr: '',
    signature: '',
    jsApiList: []
  })
  window.wx.ready(async () => {
    await new Promise(resolve => setTimeout(resolve, 200)) // 立刻resolve的话，自动播放可能会失败
    resolve('wx')
  })
})

export class AudioPromiseInner {
  async play(src, startTime=0, endTime=undefined, timeUpdate=f=>f, updateStatus = f => f) {
    this.timeUpdateCB = timeUpdate
    if (!src) throw new Error('bad audio src: ', src)
    let r = await window.weixinReadyPromise
    if (!this.audio) {
      this.audio = new Audio()
      this.audio.addEventListener("canplay", () => this.onCanPlay())
      this.audio.addEventListener("ended", () => this.onEnded())
      this.audio.addEventListener("timeupdate", e => this.onTimeupdate(e))
      for (let e of "abort error stalled suspend waiting".split(' ')) {
        console.log('adding listener: ', e)
        this.audio.addEventListener(e, (...args) => console.log('event: ', ...args, 'for', src))
      }
    }
    if (this.src !== src) {
      if (this.src) {
        this.pause('', 'paused')
      }
      this.updateStatus = updateStatus
      this.src = src
      if (src) {
        this.audio.src = src
        this.audio.load()
      }
      this.canPlayPromise = new Promise((resolve, reject) => {
        this.canPlayDefer = {resolve, reject}
      })
    }
    console.log(`playing ${this.src} from ${startTime} -> ${endTime ||''}`)
    // alert('before canplay')
    await this.canPlayPromise
    // alert('after canplay')
    return new Promise((resolve, reject) => { // promise 返回三种状态：ended paused error
      this.defer = {resolve, reject}
      this.endTime = endTime
      this.audio.currentTime = startTime
      let p = this.audio.play()
      p && p.catch && p.catch(err => { // 错误的情况也要让播放返回，低版本iphone不支持catch
        resolve('error')
      })
    })
  }
  pause(src, status = 'paused') {
    if (src && src !== this.src)
      return
    this.audio && this.audio.pause()
    this.updateStatus && this.updateStatus('paused')
    if (this.defer) {
      this.defer.resolve(status)
    }
  }
  onEnded() {
    this.pause('', 'ended')
  }
  onTimeupdate(e) {
    if (this.endTime && +e.target.currentTime > this.endTime) {
      this.pause('', 'ended')
    }
    this.timeUpdateCB(e)
  }
  onCanPlay() {
    this.canPlayDefer.resolve(true)
  }
}

let singleAudio = new AudioPromiseInner('')

export default class AudioPromise {
  constructor(src, timeUpdate = f=>f, updateStatus=f=>f) {
    this.src = src
    this.timeUpdate = timeUpdate
    this.updateStatus = updateStatus

  }
  async play(startTime=0, endTime=undefined) {
    return singleAudio.play(this.src, startTime, endTime, this.timeUpdate, this.updateStatus)
  }
  async pause() {
    return singleAudio.pause(this.src)
  }
}
