import * as utils from './utils'

export default {
  device_name: window.navigator.userAgent.toLowerCase().match(/\((.*?)\)/)[1] || '',
  objects: {},
  alerted: false,
  logined: {},
  loginPromise: null,
  nowPromise: null,
  getCodeKey() {
    let c = this.objects.config;
    return `code-${c.appid}`
  },
  getTokenKey() {
    let c = this.objects.config;
    return `token-${c.appid}`
  },
  goAuth() {
    this.alerted = true // 禁止跳转之后的报错
    utils.cacheSet(this.getCodeKey(), '', 0)
    require('./weixin').weixinAuth(window.location.href)
    throw 'go auth'
  },
  async login(vue, force = 0) {
    if (typeof vue === 'number') force = vue // 兼容以前的调用
    return this.login2(force)
  },
  async login2(force = 0) {
    if (!this.nowPromise) {
      this.nowPromise = axios.get(`/api/mobile/tool/now`)
    }
    let {data: now} = await this.nowPromise // 从服务端获取当前时刻

    let route = this.objects.router.currentRoute
    if (!this.loginPromise && route.query.token) { // 通过url中的token来跳过登陆过程
      this.logined.token = route.query.token
      this.loginPromise = Promise.resolve(this.logined)
    }
    if (!force && this.loginPromise) {
      return await this.loginPromise
    }
    if (utils.isWeixin()) {
      return this.loginWeixin(now)
    } else {
      return this.loginOther()
    }
  },
  async loginOther() {
    console.log('loginOther')
    this.logined.token = utils.cacheGet(this.getTokenKey()) || ''
    if (this.logined.token) {
      let {data} = await axios.post(`/api/mobile/user/login/loginByPhoneToken`, {
        includes: 'vip',
        app_name: this.objects.config.app_name,
        token: this.logined.token,
      })
      Object.assign(this.logined, data)
    }
    return this.logined
  },
  loginWeixin(now) {
    let route = this.objects.router.currentRoute
    this.objects.config.appid = route.query.appid || this.objects.config.appid
    let code = route.query.code
    let timestamp = route.query._ts || "0"
    if (!code || now - +timestamp > 30) {
      code = utils.cacheGet(this.getCodeKey())
    }
    if (!code) {
      this.goAuth()
    }
    if (code && now - +timestamp < 30) { // 只有最新获取的code才能够更新缓存时间
      utils.cacheSet(this.getCodeKey(), code, 86400)
    }
    this.loginPromise = this.loginWeixinInner(code)
    return this.loginPromise
  },
  async loginWeixinInner(code) {
    let {data} = await axios.post(`/api/mobile/user/login/loginByCode`, {
      code,
      device_name: this.device_name,
    }, {disableAlert: true})
    if (data.code === 'WX_LOGIN_ERROR') { // 微信在登陆过程中，偶尔发生system error，只能够再次授权登陆
      this.goAuth()
    }
    Object.assign(this.logined, data)
    return data
  },

  async bindPhone(options) {
    let {data} = await axios.post(`/api/mobile/user/login/${utils.isWeixin() ? 'wechatBindPhone' : 'loginBySmsCode'}`, {
      ...options,
      includes: 'vip',
      app_name: this.objects.config.app_name,
      device_name: this.device_name,
      code: utils.cacheGet(this.getCodeKey())
    })
    if (data.code) {
      this.objects.Mint.Toast(data.message)
    } else {
      Object.assign(this.logined, data)
    }
    utils.cacheSet(this.getTokenKey(), this.logined.token)
  },
  globalParameter() {
    let {config} = this.objects
    return {
      device_id: utils.getDeviceId(),
      uid: this.logined.uid,
      token: this.logined.token,
      version: config.ver,
      appid: config.appid,
      openid: this.logined.openid,
      app_name: config.app_name,
    }
  },
}
